﻿using System;
using System.Collections.Generic;
using Microsoft.Web.Services3;
using Microsoft.Web.Services3.Design;
using Microsoft.Web.Services3.Security;
using Microsoft.Web.Services3.Security.Tokens;
using System.Security.Cryptography.X509Certificates;
using System.Xml;

namespace LIB2BWSSample
{
    public class OBWSClientAssertion : SecurityPolicyAssertion
    {
        public X509TokenProvider ServiceX509TokenProvider;
        public X509TokenProvider ClientX509TokenProvider;

        public OBWSClientAssertion()
        {
        }

        public void setClientCertificate(StoreLocation storeLocation, StoreName storeName, String subjectName)
        {
            ClientX509TokenProvider = new X509TokenProvider(storeLocation, storeName, subjectName);
        }

        public void setServiceCertificate(StoreLocation storeLocation, StoreName storeName, String subjectName)
        {
            ServiceX509TokenProvider = new X509TokenProvider(storeLocation, storeName, subjectName);
        }

        public override SoapFilter CreateClientInputFilter(FilterCreationContext context)
        {
            return new ClientReceiveFilter(this, context);
        }

        public override SoapFilter CreateClientOutputFilter(FilterCreationContext context)
        {
            return new ClientSendFilter(this, context);
        }

        public override SoapFilter CreateServiceInputFilter(FilterCreationContext context)
        {
            return null;
        }

        public override SoapFilter CreateServiceOutputFilter(FilterCreationContext context)
        {
            return null;
        }
    }

    class ClientSendFilter : SendSecurityFilter
    {
        OBWSClientAssertion parent;
        public ClientSendFilter(OBWSClientAssertion parentAssertion, FilterCreationContext filterContext)
            : base(parentAssertion.ServiceActor, false, parentAssertion.ClientActor)
        {
            parent = parentAssertion;
        }

        public override void SecureMessage(SoapEnvelope envelope, Security security)
        {
            // If this line throws an exception you are not providing the right cert subject
            SecurityToken tokenClient = parent.ClientX509TokenProvider.GetToken();

            MessageSignature msgSig = new MessageSignature();
            msgSig.SigningToken = tokenClient;
            security.Tokens.Add(tokenClient);

            security.Elements.Add(msgSig);
            security.Tokens.Add(envelope.Context.IdentityToken);
        }
    }

    class ClientReceiveFilter : ReceiveSecurityFilter
    {
        OBWSClientAssertion parent;

        public ClientReceiveFilter(OBWSClientAssertion parentAssertion, FilterCreationContext filterContext)
            : base(parentAssertion.ServiceActor, false, parentAssertion.ClientActor)
        {
            parent = parentAssertion;
        }

        public override void ValidateMessageSecurity(SoapEnvelope envelope, Security security)
        {
            bool signed = false;
            if (security == null)
                return;

            foreach (ISecurityElement elem in security.Elements)
            {
                if (elem is MessageSignature)
                {
                    MessageSignature sig = (MessageSignature)elem;
                    if (sig.SigningToken.Equals(parent.ServiceX509TokenProvider.GetToken()))
                        signed = true;
                }
            }

            if (!signed)
                throw new Exception("Response message does not meet security requirements");
        }
    }
}