﻿using System;
using LIB2BWSSample.B2BWS.Statements;
using System.Security.Cryptography.X509Certificates;
using Microsoft.Web.Services3.Addressing;
using Microsoft.Web.Services3.Security.Tokens;
using Microsoft.Web.Services3.Design;

namespace LIB2BWSSample
{
    public class Program
    {
        static OBWSClientAssertion assertion = new OBWSClientAssertion();

        static void Main(string[] args)
        {
            /*******************************************************
             * 29-01-2018 G.Þ.B.
             * 
             * This is a small simple test program to help the customers 
             * of Landsbankinn connecting to our B2BWS (IOBS) web service.
             * This should be regarded as a starting point for programmers 
             * and could be turned into a library.
             * 
             * The example below only connects to the Statements endpoint and
             * fetches yesterdays exchange rate. 
             * 
             * If you want to do this yourself:
             * The hardest part is to make the service references. You'll 
             * need to install WSE 3.x and the WSE VS addon and use that to 
             * create the *Wse classes (i.e. the ones that end in Wse like the 
             * StatementServiceWse below). There is some literature about using 
             * WSE with newer versions of VS online. The latest version we 
             * managed to get working using the WSE VS addon was VS2013.
             * After the services references are ready it should be pretty 
             * straight forward and only the WSE3 dll is needed from then on.
             * 
             * More reading about using the WSE VS addon:
             * https://digantakumar.com/2010/06/04/wse-3-in-visual-studio-2008-and-2010/
             * 
             * Provided as-is.
             *
             *******************************************************/

            // Fetch parameters from config
            string username = System.Configuration.ConfigurationManager.AppSettings["Username"];
            string password = System.Configuration.ConfigurationManager.AppSettings["Password"];
            string serverUri = System.Configuration.ConfigurationManager.AppSettings["ServerUri"];
            string serviceCertificateName = System.Configuration.ConfigurationManager.AppSettings["ServiceCertificateName"];
            string clientCertificateName = System.Configuration.ConfigurationManager.AppSettings["ClientCertificateName"];
            
            // Init the service
            IcelandicOnlineBankingStatementsServiceWse statementsService = new IcelandicOnlineBankingStatementsServiceWse();
            statementsService.Destination = new EndpointReference(new Uri("https://b2bws.fbl.is/statements.asmx"), new Uri(serverUri + "/Statements.asmx"));

            // Set the certificates, policy and login information
            assertion.setServiceCertificate(StoreLocation.CurrentUser, StoreName.TrustedPeople, serviceCertificateName);
            assertion.setClientCertificate(StoreLocation.CurrentUser, StoreName.My, clientCertificateName);

            UsernameToken userToken = new UsernameToken(username, password, PasswordOption.SendPlainText);
            statementsService.SetClientCredential(userToken);

            Policy policy = new Policy();
            policy.Assertions.Add(assertion);

            statementsService.SetPolicy(policy);
            statementsService.RequestSoapContext.IdentityToken = userToken;
      
            // Get the current currency exchange rate
            CurrencyRate[] currencyRate = statementsService.GetCurrencyRates(CurrencyType.ExchangeRate, DateTime.Now.AddDays(-1));

            if (!(currencyRate.Length > 0))
            {
                Console.WriteLine("Did not get any currency rate from B2BWS!");
            }
            else
            {
                Console.WriteLine("Got currency rate from B2BWS: " + currencyRate[0].CurrencyCode + " " + currencyRate[0].BuyingRate + " " + currencyRate[0].SellingRate);
            }

            Console.ReadKey();
        }
    }
}
